/*
	projekt:	numitron_clock.c
	datum:		30.05.2013
	autor:		KOSTKA Filip
				SENFLUK Petr

	*****************************
	Numitronove hodiny s Atmega32
	*****************************

	obsluha:	pin PD5 - PWM
				pin PB2 - zhaveni
				
	zobrazovace:	SEG1 - PD7
					SEG2 - PD6
					SEG3 - PB3
					SEG4 - PD4
					SEG5 - PB0
					SEG6 - PB1
	
	segmenty: PAx
	   _____
      |  1  |
    3 |_____| 0
	  |  2  |
    4 |_____| 7 # 6
	     5

*/

#define F_CPU 14745600

#include <avr/io.h>
#include <avr/interrupt.h>

	//definice znakove sady
	const char dNULL = 0b00000000;
	const char dDASH = 0b00000100;
	const char dTOP = 0b00000010;
	const char dBOTTOM = 0b00100000;
	
	const char d0 = 0b10111011;
	const char d1 = 0b10000001;
	const char d2 = 0b00110111;
	const char d3 = 0b10100111;
	const char d4 = 0b10001101;
	const char d5 = 0b10101110;
	const char d6 = 0b10111110;
	const char d7 = 0b10000011;
	const char d8 = 0b10111111;
	const char d9 = 0b10101111;
	
	const char dA = 0b10011111;
	const char db = 0b10111100;
	const char dE = 0b00111110;
	const char dJ = 0b10100001;
	const char dS = 0b10101110;
	const char dY = 0b10001101;

	const char menu_top = 6;
	const char brigh_top = 3;

	//stavove promenne
	char time_update, set_update, brigh_update = 1;
	char menu_state = 0, mpx_state = 1, brigh_state = 3;
	char hr = 0; char min = 0; char sec = 0;
	char digit_1, digit_2, digit_3, digit_4, digit_5, digit_6;
	char time_cnt;

//tlacitko MENU
ISR(INT0_vect ){
	if(menu_state == menu_top)
		menu_state = 0;
	else
		menu_state += 1;
}

//tlacitko SET
ISR(INT1_vect ){
	set_update = 1;
}

//citac OC0 - multiplex
ISR(TIMER0_OVF_vect ){
	//frekvence spousteni 900Hz
	if(mpx_state == 6)
		mpx_state = 1;
	else
		mpx_state += 1;
}

//citac OC2 - casomira
ISR(TIMER2_OVF_vect ){
	//frekvecne spousteni 225Hz
	if(time_cnt == 225){
		time_cnt = 0;
		time_update = 1;
	}
	else
		time_cnt += 1;
}

char num_to_digit(short num){
	switch(num){
		case 0 : return d0; 
		case 1 : return d1;
		case 2 : return d2;
		case 3 : return d3;
		case 4 : return d4;
		case 5 : return d5;
		case 6 : return d6;
		case 7 : return d7;
		case 8 : return d8;
		case 9 : return d9;
	}
	return dDASH;	//pro pripad spatneho vstupniho parametru
}

//inicializace casu
void time_init(){
	if(sec < 59)
		sec += 1;
	else{
		sec = 0;
		if(min < 59)
			min += 1;
		else{
			min = 0;
			if(hr < 23)
				hr += 1;
			else
				hr = 0;
		}//min
	}//sec
	time_update = 0;
}

//inicializace pri stisknuti tlacitka SET
void set_init(){
	switch(menu_state){
		case 0 : ; break;	//rezerva (datum / sleep mode ...)
		case 1 : if(hr == 23)
				 	hr = 0;
				 else
				 	hr += 1; break;
		case 2 : if(min == 59)
				 	min = 0;
				 else
				 	min += 1; break;
		case 3 : if(brigh_state == brigh_top)
				 	brigh_state = 0;
				 else
				 	brigh_state += 1;
				 brigh_update = 1; break;
	}//switch
	set_update = 0;
}

//menu_state = 0 - IDLE
void menu_0(){
	digit_1 = num_to_digit(hr / 10);
	digit_2 = num_to_digit(hr % 10) | 0b01000000;
	digit_3 = num_to_digit(min / 10);
	digit_4 = num_to_digit(min % 10) | 0b01000000;
	digit_5 = num_to_digit(sec / 10);
	digit_6 = num_to_digit(sec % 10) | 0b01000000;
}

//menu_state = 1 - NASTAVENI HODIN
void menu_1(){
	digit_1 = num_to_digit(hr / 10);
	digit_2 = num_to_digit(hr % 10) | 0b01000000;
	digit_3 = dDASH;
	digit_4 = dDASH;
	digit_5 = dDASH;
	digit_6 = dDASH;
}

//menu_state = 2 - NASTAVENI MINUT
void menu_2(){
	digit_1 = dDASH;
	digit_2 = dDASH;
	digit_3 = num_to_digit(min / 10);
	digit_4 = num_to_digit(min % 10) | 0b01000000;
	digit_5 = dDASH;
	digit_6 = dDASH;
}

//menu_state = 3 - NASTAVENI JASU
void menu_3(){
	digit_1 = dJ;
	digit_2 = dA;
	digit_3 = dS;
	digit_4 = dNULL;
	switch(brigh_state){
		case 0 : digit_5 = d0;
				 digit_6 = d0; break;
		case 1 : digit_5 = d0;
				 digit_6 = d1; break;
		case 2 : digit_5 = d1;
				 digit_6 = d0; break;
		case 3 : digit_5 = d1;
				 digit_6 = d1; break;
	}//switch
}

//menu_state = 4 - BEZICI TEXT
void menu_4(){
	switch(sec % 6){
		case 0 : digit_1 = dDASH;
				 digit_2 = dDASH;
				 digit_3 = dDASH;
				 digit_4 = db;
				 digit_5 = dY;
				 digit_6 = dE; break;
		case 1 : digit_1 = dDASH;
				 digit_2 = dDASH;
				 digit_3 = db;
				 digit_4 = dY;
				 digit_5 = dE;
				 digit_6 = dDASH; break;
		case 2 : digit_1 = dDASH;
				 digit_2 = db;
				 digit_3 = dY;
				 digit_4 = dE;
				 digit_5 = dDASH;
				 digit_6 = dDASH; break;
		case 3 : digit_1 = db;
				 digit_2 = dY;
				 digit_3 = dE;
				 digit_4 = dDASH;
				 digit_5 = dDASH;
				 digit_6 = dDASH; break;
		case 4 : digit_1 = dY;
				 digit_2 = dE;
				 digit_3 = dDASH;
				 digit_4 = dDASH;
				 digit_5 = dDASH;
				 digit_6 = db; break;
		case 5 : digit_1 = dE;
				 digit_2 = dDASH;
				 digit_3 = dDASH;
				 digit_4 = dDASH;
				 digit_5 = db;
				 digit_6 = dY; break;
	}//switch
}

//menu_state = 5 - VLNKY
void menu_5(){
	switch((time_cnt / 15) % 4){
		case 0 : digit_1 = dBOTTOM;
				 digit_2 = dDASH;
				 digit_3 = dTOP;
				 digit_4 = dDASH;
				 digit_5 = dBOTTOM;
				 digit_6 = dDASH; break;
		case 1 : digit_1 = dDASH;
				 digit_2 = dTOP;
				 digit_3 = dDASH;
				 digit_4 = dBOTTOM;
				 digit_5 = dDASH;
				 digit_6 = dTOP; break;
		case 2 : digit_1 = dTOP;
				 digit_2 = dDASH;
				 digit_3 = dBOTTOM;
				 digit_4 = dDASH;
				 digit_5 = dTOP;
				 digit_6 = dDASH; break;
		case 3 : digit_1 = dDASH;
				 digit_2 = dBOTTOM;
				 digit_3 = dDASH;
				 digit_4 = dTOP;
				 digit_5 = dDASH;
				 digit_6 = dBOTTOM; break;
	}//switch
}

//menu_state = 6 - BINARNI SEKUNDOVKA
void menu_6(){
	char temp = sec;
	digit_6 = num_to_digit(temp % 2);
	temp  /= 2;
	digit_5 = num_to_digit(temp % 2);
	temp  /= 2;
	digit_4 = num_to_digit(temp % 2);
	temp  /= 2;
	digit_3 = num_to_digit(temp % 2);
	temp  /= 2;
	digit_2 = num_to_digit(temp % 2);
	temp  /= 2;
	digit_1 = num_to_digit(temp % 2);
	temp  /= 2;
}

int main(void ){
	
	//globalni preruseni
	sei();
	
	//preruseni pro tlacitka
	GICR |= (1 << INT1)|(1 << INT0);	//external interrupt
	MCUCR |= (1 << ISC01)|(1 << ISC11);	//falling edge
	
	//preruseni citace OC0 - multiplex
	TIMSK |= (1 << TOIE0);
	
	//preruseni citace OC2 - casomira
	TIMSK |= (1 << TOIE2);
	
	//nastaveni citace 0C0 - multiplex
	TCCR0 |= (1 << CS00)|(1 << CS01);	//prescaler 64

	//nastaveni citace OC1A - PWM
	TCCR1A |= (1 << COM1A1);			//set at bottom, clear on match
	TCCR1A |= (1 << WGM10);				//fast PWM 8-bit
	TCCR1B |= (1 << WGM12);
	TCCR1B |= (1 << CS10);				//no prescaler
	
	//nastaveni citace 0C2 - casomira
	TCCR2 |= (1 << CS22)|(1 << CS21);	//prescaler 256
	
	//inicializece portu - celkem 16 vystupnich pinu
	DDRA = 0b11111111;
	DDRB = 0b00001111;
	DDRC = 0b00000000;
	DDRD = 0b11110000;
	
	//zhaveni (PB2), pull-up na INT0 (PD2), INT2 (PD3) a PWM (PD5)
	PORTB = 0b00000100;
	PORTD = 0b00101100;
	
	while(1){
		//...externi aktualizace - tlacitko SET
		if (set_update == 1){
			set_init();
		}

		//...nastaveni PWM
		if(brigh_update == 1){
			switch(brigh_state){
				case 0 : OCR1AL = 102; break;	//...2/5 Ucc
				case 1 : OCR1AL = 154; break;	//...3/5 Ucc
				case 2 : OCR1AL = 205; break;	//...4/5 Ucc
				case 3 : OCR1AL = 255; break;	//...Ucc
			}//switch
			brigh_update = 0;
		}

		//...interni aktualizace - cas
		if (time_update == 1){
			time_init();
		}
		
		//...sestaveni displeje
		switch(menu_state){
			case 0 : menu_0(); break;	//IDLE
			case 1 : menu_1(); break;	//NASTAVENI HODIN
			case 2 : menu_2(); break;	//NASTAVENI MINUT
			case 3 : menu_3(); break;	//NASTAVENI JASU
			case 4 : menu_4(); break;	//BEZICI TEXT - ukazka
			case 5 : menu_5(); break;	//VLNY - ukazka
			case 6 : menu_6(); break;	//BINARNI SEKUNDOVKA - ukazka
		}//switch

		//...multiplex
		switch(mpx_state){
			case 1: PORTB &= ~0b00000010;	//vypnuti katody 6. zobrazovace
					PORTD |= 0b10000000;	//zapnuti katody 1. zobrazovace
					PORTA = digit_1; break;	//odeslani znaku
			case 2: PORTD &= ~0b10000000;	//...
					PORTD |= 0b01000000;
					PORTA = digit_2; break;
			case 3: PORTD &= ~0b01000000;
					PORTB |= 0b00001000;
					PORTA = digit_3; break;
			case 4: PORTB &= ~0b00001000;
					PORTD |= 0b00010000;
					PORTA = digit_4; break;
			case 5: PORTD &= ~0b00010000;
					PORTB |= 0b00000001;
					PORTA = digit_5; break;
			case 6: PORTB &= ~0b00000001;
					PORTB |= 0b00000010;
					PORTA = digit_6; break;
		}
	}//while

}//main
